import asyncio
import os
import signal

from lmk.processmon.attach import attach
from lmk.processmon.client import send_signal
from lmk.processmon.child_monitor import ChildMonitor
from lmk.processmon.daemon import ProcessMonitorController, ProcessMonitorDaemon
from lmk.processmon.manager import JobManager
from lmk.utils import setup_logging, shutdown_loop


def get_interrupt_action() -> str:
    while True:
        try:
            input_value = input("interrupt/detach/resume process (i/d/r): ").lower().strip()
            if input_value in {"i", "d", "r"}:
                return input_value
        except KeyboardInterrupt:
            return "d"
        else:
            print(f"Invalid selection: {input_value}")


def main() -> None:
    setup_logging()

    base_path = os.path.expanduser("~/.lmk")
    manager = JobManager(base_path)

    monitor = ChildMonitor(["python", "test.py"])

    job = manager.create_job("python")
    print("JOB ID", job.job_id)
    socket_path = os.path.join(job.job_dir, "daemon.sock")

    controller = ProcessMonitorController(
        -1,
        monitor,
        job.job_dir,
    )
    process = ProcessMonitorDaemon(
        controller,
        job.pid_file
    )
    process.start()

    loop = asyncio.get_event_loop()
    attachment = loop.run_until_complete(attach(job.job_dir))

    interupts = 0
    while True:
        task = loop.create_task(attachment.wait())
        try:
            loop.run_until_complete(task)
            break
        except KeyboardInterrupt:
            task.cancel()

            if interupts > 0:
                loop.run_until_complete(attachment.stop())
                loop.run_until_complete(shutdown_loop(loop))
                break

            attachment.pause()
            action = get_interrupt_action()
            if action == "i":
                attachment.resume()
                loop.run_until_complete(send_signal(socket_path, signal.SIGINT))
                interupts += 1

            if action == "d":
                loop.run_until_complete(attachment.stop())
                loop.run_until_complete(shutdown_loop(loop))
                break
            
            if action == "r":
                attachment.resume()


if __name__ == "__main__":
    main()



from typing import Callable, Tuple, Any


def stack_decorators(
    *decs: Callable[..., Any]
) -> Callable[[Callable[..., Any]], Callable[..., Any]]:
    def dec(f):
        for func in reversed(decs):
            f = func(f)
        return f

    return dec



# coding: utf-8

# flake8: noqa
"""
    LMK API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

# import models into model package
from lmk.generated.models.access_token_request import AccessTokenRequest
from lmk.generated.models.access_token_response import AccessTokenResponse
from lmk.generated.models.actor_response import ActorResponse
from lmk.generated.models.app_response import AppResponse
from lmk.generated.models.create_headless_auth_session_request import CreateHeadlessAuthSessionRequest
from lmk.generated.models.create_session_request import CreateSessionRequest
from lmk.generated.models.create_session_request_state import CreateSessionRequestState
from lmk.generated.models.email_channel_payload import EmailChannelPayload
from lmk.generated.models.event_notification_configuration import EventNotificationConfiguration
from lmk.generated.models.event_request import EventRequest
from lmk.generated.models.event_response import EventResponse
from lmk.generated.models.headless_auth_refresh_token_request import HeadlessAuthRefreshTokenRequest
from lmk.generated.models.headless_auth_session_response import HeadlessAuthSessionResponse
from lmk.generated.models.jupyter_session_state import JupyterSessionState
from lmk.generated.models.none_session_state import NoneSessionState
from lmk.generated.models.notification_channel_info import NotificationChannelInfo
from lmk.generated.models.notification_channel_response import NotificationChannelResponse
from lmk.generated.models.notification_channel_response_payload import NotificationChannelResponsePayload
from lmk.generated.models.notification_channels_response import NotificationChannelsResponse
from lmk.generated.models.partial_jupyter_session_state import PartialJupyterSessionState
from lmk.generated.models.partial_none_session_state import PartialNoneSessionState
from lmk.generated.models.partial_process_session_state import PartialProcessSessionState
from lmk.generated.models.process_send_signal_request import ProcessSendSignalRequest
from lmk.generated.models.process_session_state import ProcessSessionState
from lmk.generated.models.session_action_request import SessionActionRequest
from lmk.generated.models.session_action_request_body import SessionActionRequestBody
from lmk.generated.models.session_response import SessionResponse
from lmk.generated.models.text_message_channel_payload import TextMessageChannelPayload
from lmk.generated.models.update_session_request import UpdateSessionRequest
from lmk.generated.models.update_session_request_state import UpdateSessionRequestState
from lmk.generated.models.web_push_channel_payload import WebPushChannelPayload



# coding: utf-8

"""
    LMK API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by: https://openapi-generator.tech
"""


from __future__ import annotations
from inspect import getfullargspec
import pprint
import re  # noqa: F401
import json


from typing import Any, Optional
from pydantic import BaseModel, Field, StrictStr, validator
from lmk.generated.models.actor_response import ActorResponse
from lmk.generated.models.create_session_request_state import CreateSessionRequestState

class SessionResponse(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """
    state: CreateSessionRequestState = ...
    session_id: StrictStr = Field(..., alias="sessionId")
    name: StrictStr = ...
    type: StrictStr = ...
    created_at: StrictStr = Field(..., alias="createdAt")
    created_by_actor: ActorResponse = Field(..., alias="createdByActor")
    last_updated_at: StrictStr = Field(..., alias="lastUpdatedAt")
    last_updated_by_actor: ActorResponse = Field(..., alias="lastUpdatedByActor")
    ended_at: Optional[StrictStr] = Field(..., alias="endedAt")
    ended_by_actor: Optional[Any] = Field(..., alias="endedByActor")
    __properties = ["state", "sessionId", "name", "type", "createdAt", "createdByActor", "lastUpdatedAt", "lastUpdatedByActor", "endedAt", "endedByActor"]

    @validator('type')
    def type_validate_enum(cls, v):
        if v not in ('jupyter', 'process', 'none'):
            raise ValueError("must validate the enum values ('jupyter', 'process', 'none')")
        return v

    class Config:
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SessionResponse:
        """Create an instance of SessionResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of state
        if self.state:
            _dict['state'] = self.state.to_dict()
        # override the default output from pydantic by calling `to_dict()` of created_by_actor
        if self.created_by_actor:
            _dict['createdByActor'] = self.created_by_actor.to_dict()
        # override the default output from pydantic by calling `to_dict()` of last_updated_by_actor
        if self.last_updated_by_actor:
            _dict['lastUpdatedByActor'] = self.last_updated_by_actor.to_dict()
        # override the default output from pydantic by calling `to_dict()` of ended_by_actor
        if self.ended_by_actor:
            _dict['endedByActor'] = self.ended_by_actor.to_dict()
        # set to None if ended_at (nullable) is None
        if self.ended_at is None:
            _dict['endedAt'] = None

        # set to None if ended_by_actor (nullable) is None
        if self.ended_by_actor is None:
            _dict['endedByActor'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> SessionResponse:
        """Create an instance of SessionResponse from a dict"""
        if obj is None:
            return None

        if type(obj) is not dict:
            return SessionResponse.parse_obj(obj)

        _obj = SessionResponse.parse_obj({
            "state": CreateSessionRequestState.from_dict(obj.get("state")) if obj.get("state") is not None else None,
            "session_id": obj.get("sessionId"),
            "name": obj.get("name"),
            "type": obj.get("type"),
            "created_at": obj.get("createdAt"),
            "created_by_actor": ActorResponse.from_dict(obj.get("createdByActor")) if obj.get("createdByActor") is not None else None,
            "last_updated_at": obj.get("lastUpdatedAt"),
            "last_updated_by_actor": ActorResponse.from_dict(obj.get("lastUpdatedByActor")) if obj.get("lastUpdatedByActor") is not None else None,
            "ended_at": obj.get("endedAt"),
            "ended_by_actor": ActorResponse.from_dict(obj.get("endedByActor")) if obj.get("endedByActor") is not None else None
        })
        return _obj




from lmk.jupyter.widget import get_widget


__all__ = ["set_monitoring_state", "set_log_level"]


def __getattr__(name: str):
    if name not in __all__:
        raise AttributeError(name)
    widget = get_widget()
    return getattr(widget, name)



import asyncio
import os
import sys

from lmk.processmon.attach import attach_simple
from lmk.utils import setup_logging


async def main():
    setup_logging()
    job_id = sys.argv[1]
    job_dir = os.path.expanduser(f"~/.lmk/jobs/{job_id}")
    await attach_simple(job_dir)


if __name__ == "__main__":
    asyncio.run(main())
